{% load i18n %}


<p class="paginator">
    {% if first_page_url %}
        <a href="{{ first_page_url }}">1</a>
    {% endif %}
    {% if prev_page_url %}
        <a href="{{ prev_page_url }}">&larr;</a>
    {% endif %}
    {% if cur_number == 1 and not cl.result_list %}
        0 {{ cl.opts.verbose_name_plural }}
    {% else %}
        <span class="this-page">{% trans 'Page' %} {{ cur_number }}</span>
    {% endif %}
    {% if next_page_url %}
        <a href="{{ next_page_url }}">&rarr;</a>
    {% endif %}
    {% if cl.formset and cl.result_count %}<input type="submit" name="_save" class="default" value="{% trans 'Save' %}">{% endif %}
</p>